package widebase.rq.concurrent

import java.util.concurrent.LinkedBlockingQueue

import org.msgpack.rpc.Request

/** A consumer on server side.
 *
 * @author myst3r10n
 */
class Consumer(
  val authId: String,
  val table: String,
  selector: String,
  interval: Int = 0)
  extends Thread {

  /** Holds pending request. */
  protected val nextRequest = new LinkedBlockingQueue[Request]

  /** Holds pending records. */
  protected val nextRecords = new LinkedBlockingQueue[Array[Byte]]

  /** Holds rollback information. */
  object rollback {

    var records = 0
    var partition = ""

  }

  /** Add end of day. */
  def endOfDay {

    nextRecords.add(Array[Byte]())

  }

  /** Add pending request. */
  def next(request: Request) {

    nextRequest.add(request)

  }

  override def run {

    try {

      var request: Request = null
      var records: Array[Byte] = null

      while(true) {

        if(interval > 0)
          Thread.sleep(interval)

        request = nextRequest.take
        records = nextRecords.take

        // Selector code...

        request.sendResult(records)

      }

    } catch {
      case e: InterruptedException => // Subscriber stopped.

    }
  }

  /** Add pending upsert. */
  def upsert(bytes: Array[Byte]) {

    nextRecords.add(bytes)

  }

  /** Shutdown consumer. */
  def shutdown {

    if(!this.isInterrupted)
      this.interrupt

  }
}

